var StringTie = function()
{
	var stringtie = {};
	var debug = 0;
	var subtypeAlignedSequences = null;
	var fileTypeBAM = null;
	
	var clusterIsValid = false;
	var arrayDesignIsValid = false;
	
	// Page initialization
	stringtie.initPage = function()
	{
		
		// Step 1
		Buttons.addClickHandler('btnSelectAlignedSequences', stringtie.selectAlignedSequences);
		Events.addEventHandler('alignedSequences', 'base-selected', stringtie.setAlignedSequenceCallback);
		Events.addEventHandler('step-1', 'wizard-validate', stringtie.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', stringtie.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', stringtie.validateStep2);
		Events.addEventHandler('clusters', 'change', stringtie.clusterOnChange);
		Events.addEventHandler('stringtieSoftware', 'change', stringtie.softwareOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', stringtie.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../StringTie.servlet?ID='+App.getSessionId();
		url += '&cmd=GetAlignedSequencesForStringTie';
		Wizard.asyncJsonRequest(url, stringtie.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	stringtie.initializeStep1 = function(response)
	{
		var alignedSequences = response.alignedSequences;
		var frm = document.forms['reggie'];	

		if (alignedSequences != null && alignedSequences.length > 0)
		{
			var firstPlate = null;
			for (var asNo=0; asNo < alignedSequences.length; asNo++)
			{
				var as = alignedSequences[asNo];
				var lib = as.lib;
				var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
				var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
				if (firstPlate == null && libPlate != null) firstPlate = libPlate.name;
				
				var name = (asNo+1) + ': ';
				if (libPlate) name += libPlate.name + ' - ';
				name += as.name;
				if (as.AutoProcess == 'ReProcess')
				{
					name += ' [R]';
				}
				
				var option = new Option(name, as.id, false, libPlate != null && libPlate.name==firstPlate);
				if (isYellow) option.className = 'yellow';
				option.alignedSequences = as;
				frm.alignedSequences.options[frm.alignedSequences.length] = option;
			}
		}
		else
		{
			Wizard.setInputStatus('alignedSequences', 'invalid', 'No new sequences available for processing');
		}
		
		Doc.show('step-1');
		Doc.show('gonext');

		frm.alignedSequences.focus();
	}
	
	stringtie.validateStep1 = function(event)
	{
		Wizard.setInputStatus('alignedSequences', '', '');
		var frm = document.forms['reggie'];
		for (var asNo = 0; asNo < frm.alignedSequences.length; asNo++)
		{
			if (frm.alignedSequences[asNo].selected) return;
		}
		Wizard.setInputStatus('alignedSequences', 'invalid', 'Select at least one item');
		event.preventDefault();
	}
	
	stringtie.selectAlignedSequences = function()
	{
		var frm = document.forms['reggie'];
		if (frm.alignedSequences.disabled) return;
		
		if (subtypeAlignedSequences == null)
		{
			subtypeAlignedSequences = Reggie.getSubtypeInfo('ALIGNED_SEQUENCES');
		}
		if (fileTypeBAM == null)
		{
			fileTypeBAM = Reggie.getDataFileTypeInfo('BAM');
		}
		
		var url = '&resetTemporary=1';
		// Alignments with Hisat gives us 'AlignedSequences' items with 'alignment.bam'
		url += '&tmpfilter:INT:itemSubtype='+subtypeAlignedSequences.id;
		url += '&tmpfilter:STRING:'+encodeURIComponent('¤'+fileTypeBAM.id+'(file.name)')+'=alignment.bam';

		Dialogs.selectItem('DERIVEDBIOASSAY', 'alignedSequences', 1, url);
	}

	stringtie.setAlignedSequenceCallback = function(event)
	{
		var ms = event.detail;
		
		var frm = document.forms['reggie'];
		var alignedSequences = frm.alignedSequences;
		for (var i = 0; i < alignedSequences.length; i++)
		{
			if (alignedSequences[i].value == ms.id)
			{
				alignedSequences[i].selected = true;
				Wizard.setInputStatus('alignedSequences');
				return;
			}
		}

		var ms = event.detail;
		var option = new Option(ms.name, ms.id);
		option.alignedSequences = ms;
		option.selected = true;
		alignedSequences[alignedSequences.length] = option;
		Wizard.setInputStatus('alignedSequences');
	}

	
	// --- Step 2 -----------------------------------
	stringtie.initializeStep2 = function()
	{
		Wizard.setCurrentStep(2);
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		var frm = document.forms['reggie'];
		frm.debug.checked = debug || location.protocol != 'https:';

		Reggie.loadProtocols('FEATURE_EXTRACTION_PROTOCOL', 'stringtieProtocol', 'EXPRESSION_TYPE', 'StringTie');
		Reggie.loadSoftware('FEATURE_EXTRACTION_SOFTWARE', 'stringtieSoftware', 'EXPRESSION_TYPE,PARAMETER_SET', 'StringTie');
		
		// Load array design
		var url = '../StringTie.servlet?ID='+App.getSessionId() + '&cmd=GetArrayDesigns';
		Doc.addClass('arrayDesign', 'list-loading');
		frm.arrayDesign[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, stringtie.arrayDesignsLoaded);

		// Load open grid clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, stringtie.clustersLoaded);

		Doc.show('goregister');
		Doc.show('gocancel');
	}
	
	stringtie.validateStep2 = function(event)
	{
		if (!clusterIsValid) event.preventDefault();
		if (!arrayDesignIsValid) event.preventDefault();
	}
	
	stringtie.arrayDesignsLoaded = function(response)
	{
		Doc.removeClass('arrayDesign', 'list-loading');
		var frm = document.forms['reggie'];
		var designs = response.arrayDesigns;
		frm.arrayDesign.length = 0;
		for (var i = 0; i < designs.length; i++)
		{
			var design = designs[i];
			var option = new Option(design.name + ' ('+design.numFeatures+')', design.id, design.isDefault, design.isDefault);
			option.arrayDesign = design;
			frm.arrayDesign[frm.arrayDesign.length] = option;
		}
		if (frm.arrayDesign.length == 0)
		{
			Wizard.setInputStatus('arrayDesign', 'invalid', 'No available array designs');
			arrayDesignIsValid = false;
		}
		else
		{
			Wizard.setInputStatus('arrayDesign', 'valid');
			arrayDesignIsValid = true;
		}
	}
	
	stringtie.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.hosts;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
			Wizard.setInputStatus('clusters', 'valid');
			clusterIsValid = true;
		}
		if (frm.clusters.length == 0)
		{
			Wizard.setInputStatus('clusters', 'invalid', 'No available Open Grid clusters');
			clusterIsValid = false;
		}
		else
		{
			stringtie.clusterOnChange();
		}
	}
	
	stringtie.softwareOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var target = event.currentTarget;
		var item = target[target.selectedIndex].item;
		if (!item) item = {};
		
		Doc.element(target.id+'.parameterSet').innerHTML = Strings.encodeTags(item.ParameterSet || 'default');
		Doc.element(target.id+'.description').innerHTML = Strings.encodeTags(item.description);
		Wizard.setInputStatus(target.id, 'valid');

		if (item.ParameterSet)
		{
			var url = '../Install.servlet?ID='+App.getSessionId();
			url += '&cmd=GetParameterSetInfo';
			url += '&parameterSet='+encodeURIComponent(item.ParameterSet);
			url += '&targetId='+target.id;
			Wizard.asyncJsonRequest(url, stringtie.parameterSetInfoLoaded);
		}
	}
	
	stringtie.parameterSetInfoLoaded = function(response)
	{
		if (!response.parameters || response.parameters.length == 0)
		{
			Wizard.setInputStatus(response.targetId, 'warning', 'Can\'t find \'' + Strings.encodeTags(response.parameterSet) + '\' parameter set in reggie-config.xml');
		}
	}
	
	stringtie.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
	}

	stringtie.submit = function()
	{
		var frm = document.forms['reggie'];
			
		var submitInfo = {};
		submitInfo.stringtieSoftware = parseInt(frm.stringtieSoftware.value);
		submitInfo.stringtieProtocol = parseInt(frm.stringtieProtocol.value);
		submitInfo.arrayDesign = parseInt(frm.arrayDesign.value);
		submitInfo.cluster = frm.clusters.value;
		if (frm.priority.selectedIndex >= 0)
		{
			submitInfo.priority = parseInt(frm.priority.value);
		}
		submitInfo.debug = frm.debug.checked;
		submitInfo.autoConfirm = frm.autoConfirm.checked;
			
		var alignedSequences = [];
		submitInfo.alignedSequences = alignedSequences;
			
		for (var asNo = 0; asNo < frm.alignedSequences.length; asNo++)
		{
			if (frm.alignedSequences[asNo].selected) 
			{
				var as = {};
				as.id = frm.alignedSequences[asNo].alignedSequences.id;
				alignedSequences[alignedSequences.length] = as;
			}
		}

		var url = '../StringTie.servlet?ID='+App.getSessionId();
		url += '&cmd=StartStringTie';

		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, stringtie.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	stringtie.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return stringtie;
}();

Doc.onLoad(StringTie.initPage);

