var Pool = function()
{
	var pool = {};
	var debug = 0;

	var pools;
	var poolsAreValid = false;

	// Page initialization
	pool.initPage = function()
	{
		// Step 1
		Events.addEventHandler('bioplate', 'change', pool.bioplateOnChange);
		Events.addEventHandler('pools', 'change', pool.poolOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', pool.validateStep1);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', pool.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', pool.validateStep2);
		Events.addEventHandler('poolDate', 'blur', Wizard.validateDate);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', pool.submit);
		
		var url = '../Pool.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnprocessedPools';		
		Wizard.showLoadingAnimation('Loading pools...');
		Wizard.asyncJsonRequest(url, pool.initializeStep1);
	}


	pool.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];
		pools = response.pools;
		
		// We get a list of pools from the servlet
		// Each pool has a list of libraries 
		// We want to find the Library bioplates that are unique
		// within a pool (eg. all libraries are from the same bioplate)
		if (pools.length == 0)
		{
			Wizard.setFatalError('No pools available for processing.');
			return;
		}
		
		var poolsList = frm.pools;
		var plateList = frm.bioplate;
		var libPlates = [];
		for (var poolNo=0; poolNo < pools.length; poolNo++)
		{
			var p = pools[poolNo];
			
			// Shortcut to single-plate pools
			var libPlateForPool = p.libPlates.length == 1 && !p.isManual ? p.libPlates[0] : null;
			p.libPlate = libPlateForPool;

			// If the pool has a single library bioplate add it to the selection list
			// If the pool doesn't have a single library bioplate, nothing is added to
			// the list and the pool can only be selected when the 'manual pools' option is
			// active for the bioplates list
			if (libPlateForPool != null && !libPlates[libPlateForPool.name])
			{
				libPlates[libPlateForPool.name] = libPlateForPool;
				var plateOption = new Option(libPlateForPool.name, libPlateForPool.id);
				plateOption.plate = libPlateForPool;
				plateList[plateList.length] = plateOption;
			}

		}
		if (plateList.length > 1) plateList.selectedIndex = 1;

		pool.bioplateOnChange();
		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	
	// Add pools to the pools list based on the bioplate selection
	pool.bioplateOnChange = function()
	{
		var frm = document.forms['reggie'];
		
		var libPlateId = parseInt(frm.bioplate.value);

		// Clear current list
		frm.pools.length = 0;
		
		for (var poolNo = 0; poolNo < pools.length; poolNo++)
		{
			var p = pools[poolNo];
			// If a libplate is selected, only show pools that have all libraries from that plate
			// If no libplate is selected, only show pools that have libraries from multiple plates
			var addPool = (libPlateId &&  p.libPlate && p.libPlate.id == libPlateId) || (!libPlateId && p.libPlate == null);
			if (addPool)
			{
				var name = p.name;
				if (p.libPlate == null)
				{
					
					name += ' - ';
					var numPlates = p.libPlates.length;
					var title = '';
					if (numPlates <= 2)
					{
						for (var plateNo=0; plateNo < numPlates; plateNo++)
						{
							var libPlate = p.libPlates[plateNo];
							if (plateNo > 0) name += ', ';
							name += libPlate.name;
						}
					}
					else
					{
						name += p.libPlates[0].name + ' + ' + (numPlates-1) + ' more...';
						for (var plateNo=0; plateNo < numPlates; plateNo++)
						{
							var libPlate = p.libPlates[plateNo];
							if (plateNo > 0) title += ', ';
							title += libPlate.name;
						}
					}
				}

				var option = new Option(name, p.id, false, libPlateId || frm.pools.length == 0);
				option.pool = p;
				frm.pools[frm.pools.length] = option;
			}
		}
		
		Wizard.setInputStatus('pools');
		if (libPlateId)
		{
			frm.pools.disabled = true;
		}
		else
		{
			frm.pools.disabled = false;
		}
		pool.poolOnChange();
	}

	
	pool.poolOnChange = function()
	{
		var frm = document.forms['reggie'];
		
		poolsAreValid = false;
		Wizard.setInputStatus('pools');

		for (var i = 0; i < frm.pools.length; i++)
		{
			if (frm.pools[i].selected) poolsAreValid = true;
		}
		
		if (!poolsAreValid)
		{
			Wizard.setInputStatus('pools', 'invalid', 'Select at least one pool');
			return;
		}
		
		Wizard.setInputStatus('pools', 'valid');
	}

	pool.validateStep1 = function(event)
	{
		if (!poolsAreValid) event.preventDefault();
	}
	
	pool.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		Wizard.setCurrentStep(2);
		Doc.show('gocancel');
		Doc.show('goregister');
		
		Reggie.loadProtocols('POOLING_PROTOCOL', 'poolProtocol');

		var html = '';
		for (var i = 0; i < frm.pools.length; i++)
		{
			if (frm.pools[i].selected) 
			{
				var pool = frm.pools[i].pool;
				html += '<tr class="align-top">';
				html += '<td class="subprompt">'+Strings.encodeTags(pool.name)+'</td>';
				html += '<td class="input"><textarea rows="3" name="comments.'+pool.id+'"></textarea></td>';
				html += '<td class="status"></td>';
				html += '<td class="help"></td>';
				html += '</tr>';
			}
		}	
		Doc.element('pool-comments').innerHTML = html;
		
		for (var i = 0; i < frm.pools.length; i++)
		{
			if (frm.pools[i].selected) 
			{
				var pool = frm.pools[i].pool;
				frm['comments.'+pool.id].value = pool.comments;
			}
		}

		frm.poolDate.focus();
	}
	
	pool.validateStep2 = function(event)
	{
		if (!Wizard.isValid('poolDate')) 
		{
			event.preventDefault();
		}
	}
	
	pool.submit = function()
	{
		var frm = document.forms['reggie'];
		
		var pools = [];
		for (var i = 0; i < frm.pools.length; i++)
		{
			if (frm.pools[i].selected) 
			{
				var p = frm.pools[i].pool;
				var poolInfo = {};
				poolInfo.id = p.id;
				poolInfo.comments = frm['comments.'+p.id].value;
				pools[pools.length] = poolInfo;
			}
		}
		var submitInfo = {};
		submitInfo.pools = pools;
		submitInfo.poolDate = frm.poolDate.value;
		submitInfo.poolProtocol = parseInt(frm.poolProtocol.value, 10);
		submitInfo.poolOperator = frm.poolOperator.value;
		
		var url = '../Pool.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterPooledLibraries';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, pool.submissionResults, 'POST', JSON.stringify(submitInfo));
	}

	pool.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return pool;
}();

Doc.onLoad(Pool.initPage);

